

/**
 * The SettingsTestWidget_controller object
 * This is used to wire up the view and model with actions
 */ 
var SettingsTestWidget_controller = {
    /**
     * A dictionary of callback functions that are called whenever an
     * event is fired by this widget.
     */
    callbacks: {},

    init:function() {
        SettingsTestWidget_controller.update();
    },

    /**
     * Update the display
     */
    update:function() { 
        // Is the viewport large enough for the full view?
        var viewport = SettingsTestWidget_controller.getViewport();
        if (viewport.height <= 310  || viewport.width <= 310) {
            $.mobile.changePage($('#tile'));
        } else {
            $.mobile.changePage($('#home'));
        }
    },

   /**
    * Get the current title for this widget.
    */
    getTitle:function() {
        return "Test Widget for the Settings Template";
    },

    /**
     * Get the viewport width and height.
     * returns an object with width and height properties.
     */
    getViewport:function() {
	var e = window
	, a = 'inner';
	if ( !( 'innerWidth' in window ) ) {
	    a = 'client';
	    e = document.documentElement || document.body;
	}
	return { width : e[ a + 'Width' ] , height : e[ a + 'Height' ] }
    },

    /**
     * Register a callback function. 
     *
     * type: the name of the type of event to respond to 
     *
     * widget: [optional] the name of the widget where the event must
     * occur for the callback to be executed. This allows callbacks to
     * be register so that they will only respond to events in
     * specific widgets, or to all events.
     */
    register:function(event, callback, widget) {
	if (widget === undefined) {
	    var cbks = SettingsTestWidget_controller.callbacks[event];
	} else {
	    var cbks = SettingsTestWidget_controller.callbacks[widget + "." + event];
	}
	if (cbks === undefined) {
	    cbks = new Array();
	}
	cbks.push(callback);
	SettingsTestWidget_controller.callbacks[event] = cbks;
    },

    /**
     * Execute all the callbacks registered for a given event.
     *
     * The event object is passed directly to the callback function
     * and can contani any number of properties. Two properties that
     * important to the callback functionality are:
     *
     * type: the name of the type of event to respond to 
     *
     * widget: [optional] the name of the widget where the event
     * occured. This allows callbacks to be register so that they will
     * only respond to events in specific widgets, or to all events.
     */
    executeCallbacks:function(event) {
	// Execute all callbacks not restricted to a widget
	var cbks = SettingsTestWidget_controller.callbacks[event.type];
	if (cbks === undefined) return;
	for (var i = 0; i < cbks.length; i++) {
	    cbks[i](event);
	}

	// Execute all callbacks restricted to a widget
	if (event.widget === undefined) return;
	var cbks = SettingsTestWidget_controller.callbacks[event.widget + "." + event.type];
	if (cbks === undefined) return;
	for (var i = 0; i < cbks.length; i++) {
	    cbks[i](event);
	}
    }
};

$('#home').live('pageinit',function(event) {
   SettingsTestWidget_controller.init(); 
});

$('body').bind('orientationchange',function(event){
    SettingsTestWidget_controller.update();
})


var SettingsTestWidget_settings_controller = {
    init:function() {

    },

    /**
     * Build and show the settings page.
     * FIXME: Need to make this a JQuery function
     */
    showSettings: function(group) {
	SettingsTestWidget_settings_controller.createForm(group);
        $.mobile.changePage("#settings");
    },

    /**
     * Get the names of the available settings groups.
     * Groups are used to describe related sets of settings. When the
     * settings page is requested the user will be presented with a
     * list of links to individual settings pages for each group. If
     * no groups are defined then all settings will be displayed on a 
     * single settings page.
     * 
     * @return an array setting group names or an empty array if no groups are defined.
     */
    getGroups: function() {
	var groups = [];
	groups[0] = "about";
	groups[1] = "contact";
	return groups;
    },
	    
    /**
     * Get the settings in a given group. Note that the group names here must correspond
     * to the group names returned by SettingsTestWidget_settings_controller.getSettings()
     * 
     * @return an array containing a number of Setting objects
     */
    getSettings: function(group) {
	var settings = [];
	if (group == undefined || group == "about") {
	    settings[0] = new SettingsTestWidget_Setting("name", "Name", "The name you wish to be known by.", "text");
	    settings[1] = new SettingsTestWidget_Setting("description", "Description", "A short description that will be shown to people viewing your details.", "textarea");
	} else if (group == "contact") {
	    settings[0] = new SettingsTestWidget_Setting("homepageURL", "Homepage URL", "Your homepage URL.", "email");
	    settings[1] = new SettingsTestWidget_Setting("email", "EMail", "Your email address.", "email");
	}
	return settings;
    },
    
    /**
     * Process a settings form that has been submitted by storing all the settings
     * in widget preferences. If settings groups are being used then we only process
     * settings in that group.
     * 
     * @param group the name of the group to process if groups are being used
     */
    submitSettingsForm: function(group) {
	var settings = SettingsTestWidget_settings_controller.getSettings(group);
	var value;
	for (key in settings) {
	    value = $("#" + settings[key].id).attr("value");
	    widget.preferences.setItem(settings[key].id, value);
        };
        $.mobile.changePage("#home");
        $.mobile.loadPage( "settings.html" );
    },
	
    /**
     * Create the settings form and populate <div id="settings"> with it.
     * 
     * Groups are used to describe related sets of settings. When the
     * settings page is requested the user will be presented with a
     * list of links to individual settings pages for each group. If
     * no groups are defined then all settings will be displayed on a 
     * single settings page.
     * 
     * @param group the name of the group of settings we need a form to represent
     */
    createForm: function(group) {
	if (group == undefined) {
	    SettingsTestWidget_settings_controller.createGroupsIndex();
	} else {
	    var settings = SettingsTestWidget_settings_controller.getSettings(group);
            // FIXME: JQuerify the action	    
	    var form = $("<form action='javascript:SettingsTestWidget_settings_controller.submitSettingsForm(\"" + group + "\")'>");
			
	    var title = $("<h3>Settings: " + group + "</h3>");
	    
	    for (key in settings) {
		var label = $("<label>" + settings[key].label + "</label>");
		label.attr("for", settings[key].id);
		form.append(label);
		
		var input = $("<input>");
		input.type = settings[key].type;
		input.attr("id", settings[key].id);
		input.attr("placeholder", settings[key].placeholder)
		var value = widget.preferences.getItem(settings[key].id);
		if (value != undefined) {
		    input.attr("value", value);
		}
		form.append(input);
	        
		form.append($("<br>"));
	    };
	    
	    var submit = $("<input>");
	    submit.attr("type", "submit");
	    submit.attr("value", "Save");
	    submit.attr("class", "form-button");
	    form.append(submit);
	    
            var settingsElem = $("#settings-primary");
            settingsElem.empty();
	    title.appendTo(settingsElem);
	    form.appendTo(settingsElem).trigger("create");
	}
    },
	
    /**
     * Populate the <div id="settings" page with an index to the various settings
     * groups.
     */
    createGroupsIndex: function() {
	var groups = SettingsTestWidget_settings_controller.getGroups();
	
	var title = $("<h3>Settings Groups<h3>");
	
	var list = $("<ul data-role='listview'>");
	for (key in groups) {
            var item = $("<li><a href='javascript:SettingsTestWidget_settings_controller.showSettings(\"" + groups[key] + "\");'>" + groups[key] + "</a></li>");
	    list.append(item);
            // FIXME: JQuerify
	    // groupLink.href = "javascript:SettingsTestWidget_settings_controller.showSettings('" + groups[key] + "')"
	}
	
	var settingsElem = $("#settings-primary");
        settingsElem.empty();
	settingsElem.append(title);
	settingsElem.append(list);
    }
};

$('#home').live('pageinit',function(event) { 
  $.mobile.loadPage( "settings.html" );
  $('#showSettings').on('click',function(event){
      SettingsTestWidget_settings_controller.showSettings();
  });
});

$('#settings').live('pageinit',function(event) { 
    SettingsTestWidget_settings_controller.init();
});

/**
 * A setting to represent in the settings form.
 * 
 * @param id is used to identiy the setting and the form element created for it.
 * @param label is the text to place in the label for the form element.
 * @param placeholder is the text that will appear in the form element if the setting has not been set
 * @param type is the type of data to be stored in this setting, it is used to define the form element to use
 */
function SettingsTestWidget_Setting(id, label, placeholder, type){
    this.id = id;
    this.label = label;
    this.placeholder = placeholder
    this.type = type;
}


$('#tile').live('pageinit',function(event) { 
    $('#tile').click(function() {
        window.widget.views.openUrl("http://wookie.apache.org/widgets/CHANGEME", null, "_blank");
    });
});

