

/**
 * The assetPlayerTest_controller object
 * This is used to wire up the view and model with actions
 */ 
var assetPlayerTest_controller = {
    /**
     * A dictionary of callback functions that are called whenever an
     * event is fired by this widget.
     */
    callbacks: {},

    init:function() {
        assetPlayerTest_controller.update();
    },

    /**
     * Update the display
     */
    update:function() { 
        // Is the viewport large enough for the full view?
        var viewport = assetPlayerTest_controller.getViewport();
        if (viewport.height <= 310  || viewport.width <= 310) {
            $.mobile.changePage($('#tile'));
        } else {
            $.mobile.changePage($('#home'));
        }
    },

   /**
    * Get the current title for this widget.
    */
    getTitle:function() {
        return "Asset Player Test Widget";
    },

    /**
     * Get the viewport width and height.
     * returns an object with width and height properties.
     */
    getViewport:function() {
	var e = window
	, a = 'inner';
	if ( !( 'innerWidth' in window ) ) {
	    a = 'client';
	    e = document.documentElement || document.body;
	}
	return { width : e[ a + 'Width' ] , height : e[ a + 'Height' ] }
    },

    /**
     * Register a callback function. 
     *
     * type: the name of the type of event to respond to 
     *
     * widget: [optional] the name of the widget where the event must
     * occur for the callback to be executed. This allows callbacks to
     * be register so that they will only respond to events in
     * specific widgets, or to all events.
     */
    register:function(event, callback, widget) {
	if (widget === undefined) {
	    var cbks = assetPlayerTest_controller.callbacks[event];
	} else {
	    var cbks = assetPlayerTest_controller.callbacks[widget + "." + event];
	}
	if (cbks === undefined) {
	    cbks = new Array();
	}
	cbks.push(callback);
	assetPlayerTest_controller.callbacks[event] = cbks;
    },

    /**
     * Execute all the callbacks registered for a given event.
     *
     * The event object is passed directly to the callback function
     * and can contani any number of properties. Two properties that
     * important to the callback functionality are:
     *
     * type: the name of the type of event to respond to 
     *
     * widget: [optional] the name of the widget where the event
     * occured. This allows callbacks to be register so that they will
     * only respond to events in specific widgets, or to all events.
     */
    executeCallbacks:function(event) {
	// Execute all callbacks not restricted to a widget
	var cbks = assetPlayerTest_controller.callbacks[event.type];
	if (cbks === undefined) return;
	for (var i = 0; i < cbks.length; i++) {
	    cbks[i](event);
	}

	// Execute all callbacks restricted to a widget
	if (event.widget === undefined) return;
	var cbks = assetPlayerTest_controller.callbacks[event.widget + "." + event.type];
	if (cbks === undefined) return;
	for (var i = 0; i < cbks.length; i++) {
	    cbks[i](event);
	}
    }
};

$('#home').live('pageinit',function(event) {
   assetPlayerTest_controller.init(); 
});

$('body').bind('orientationchange',function(event){
    assetPlayerTest_controller.update();
})

/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * This provides the code for managing assets and collections of assets.
 * It provides generic functionality that is applicable to all asset types
 * and is therefore intended to be reused across all asset players.
 */
var assetPlayerTest_asset_controller = { 
    idx: 0, // index of current asset being displayed,
    auto: false, // indicates if we are auto playing assets
    staticDuration: 5000, // interval between auto play of static assets (e.g. photo's)
    interval: null, // the interval object that controls the auto player
    albums: null, // the albums of colelctions that can be played
    collection: null, // the collection of assets currently playing

    init:function() { 
	// register button events
	$('#next').click(function() {
	    assetPlayerTest_asset_controller.nextAsset();
	});

	$('#prev').click(function() {
	    assetPlayerTest_asset_controller.prevAsset();
	});

	$('#selectAlbum').click(function() {
            $.mobile.changePage($("#home"));
	});

	$('#playPause').click(function() {
	    if (!assetPlayerTest_asset_controller.auto) {
		assetPlayerTest_asset_controller.startAuto();
		// FIXME: this breaks layout - as it overwrides JQuery markup
		this.innerText = "Pause";
	    } else {
		assetPlayerTest_asset_controller.stopAuto();
		// FIXME: this breaks layout - as it overwrides JQuery markup
		this.innerText = "Play";
	    }
	});
    },

    initAlbums:function() { 
        var list = $("#albumList");
        var albums = assetPlayerTest_asset_controller.albums;
        if (albums) {
            list.empty();
            $.each(
                albums,
                function( intIndex, album ){
                    list.append(assetPlayerTest_asset_controller.getAlbumCover(album));
                    $("#" + album.title).click(function() {
                        assetPlayerTest_asset_controller.showAlbum(album);
                    });
                }
            );
        } else {
            list.append("Loading Albums...");
        }
    },

    getAlbumCover:function(album) {
      var html = "<li id='" + album.title + "'>";
      html = html + "<img src='" + album.preview + "' width='120' height='120'/> ";
      html = html + album.title;
      html = html + "</a></li>";
      return $(html);
    },

    showAlbum:function(album) {
	assetPlayerTest_asset_controller.setCollection(album);
        $.mobile.changePage("#picture");
    },

    setAlbums:function(albums) {
	assetPlayerTest_asset_controller.albums = albums;
        assetPlayerTest_asset_controller.initAlbums();
    },

    
    // Set the collection, that is the collection being shown at this time
    // to an array of assets. The "collection" should be either an array or an
    // object with an assets property which is the required array.
    //
    // The array will contain, at least, a src property which contains the src to
    // use for the HTML image.
    setCollection:function(collection){
        if (collection.assets) {
	    assetPlayerTest_asset_controller.collection = collection.assets;
        } else {
	    assetPlayerTest_asset_controller.collection = collection;
        }
        // TODO: cache images for faster slideshow. e.g. http://www.anthonymclin.com/code/7-miscellaneous/98-on-demand-image-loading-with-jquery
	assetPlayerTest_asset_controller.displayAsset(0);
    },

    getCollection:function() {
	return 	assetPlayerTest_asset_controller.collection;
    },
    
    displayAsset:function(idx) {
	var collection = assetPlayerTest_asset_controller.getCollection();
	$('#asset').attr("src", collection[idx].src);
    },

    nextAsset:function() {
	var collection = assetPlayerTest_asset_controller.getCollection();
	assetPlayerTest_asset_controller.idx += 1;
        if (assetPlayerTest_asset_controller.idx >= collection.length) { 
	    assetPlayerTest_asset_controller.idx = 0;
	};
	assetPlayerTest_asset_controller.displayAsset(assetPlayerTest_asset_controller.idx);
    },

    prevAsset:function() {
	var collection = assetPlayerTest_asset_controller.getCollection();
	assetPlayerTest_asset_controller.idx -= 1;
        if (assetPlayerTest_asset_controller.idx <0) { 
	    assetPlayerTest_asset_controller.idx = collection.length - 1;
	};
	assetPlayerTest_asset_controller.displayAsset(assetPlayerTest_asset_controller.idx);
    },

    startAuto:function() {
	assetPlayerTest_asset_controller.auto = true;
	assetPlayerTest_asset_controller.interval = 
	    window.setInterval(assetPlayerTest_asset_controller.nextAsset, 
			       assetPlayerTest_asset_controller.staticDuration);
    },

    stopAuto:function() {
	assetPlayerTest_asset_controller.auto = false;
	window.clearInterval(assetPlayerTest_asset_controller.interval);
    }
};

$('#picture').live('pageinit',function(event) { 
  assetPlayerTest_asset_controller.init(); 
});

$('#home').live('pageinit',function(event) { 
  assetPlayerTest_asset_controller.initAlbums(); 
});



/**
 * The @widget.shortname@_scanning_controller object This is used to
 * allow users using a switch device to interact efficiently with a
 * widget.
 */ 
var assetPlayerTest_scanning_controller = {
    scanning: false, // indicates if we are currently scanning
    delay: 1000, // time in milliseconds between focus change
    interval: null, // The interval object controlling the scan
    scanElements: null, // The elements to scan over
    currentElementIdx: 0, // The index of the currently scanning element

    init:function() {
        assetPlayerTest_scanning_controller.scanElements = $('[data-scanOrder]');
        if (assetPlayerTest_scanning_controller.scanning) {
	    assetPlayerTest_scanning_controller.startScanning();
        };
    },

    /**
     * Action to take when switch is released.
     * The event contains details supplied by the switch.
     */
    switchUp:function(event) {
	if (event.keyCode === 32) {
	    assetPlayerTest_scanning_controller.scanElements[assetPlayerTest_scanning_controller.currentElementIdx].click();
	}
    },

    nextElement:function() {
	var idx = assetPlayerTest_scanning_controller.currentElementIdx;
	var elements = assetPlayerTest_scanning_controller.scanElements;

	var oldElement = $(elements[idx]);
	oldElement.fadeIn('slow', function() {
	    // Animation complete.
	});

	idx += 1;
	if (idx >= elements.length) {
	    idx = 0
	};

	var newElement = $(elements[idx]);
	newElement.fadeOut('slow', function() {
	    // Animation complete.
	});
	newElement.fadeIn('slow', function() {
	    // Animation complete.
	});
	assetPlayerTest_scanning_controller.currentElementIdx = idx;
    },

    startScanning:function() {
	assetPlayerTest_scanning_controller.scanning = true;
	assetPlayerTest_scanning_controller.interval = 
	    window.setInterval(assetPlayerTest_scanning_controller.nextElement, 
			       assetPlayerTest_scanning_controller.delay);
	$(document).keyup(assetPlayerTest_scanning_controller.switchUp);
    },

    stopScanning:function() {
	assetPlayerTest_scanning_controller.scanning = false;
	window.clearInterval(assetPlayerTest_scanning_controller.interval);
    }

};

$('#home').live('pageshow',function(event) {
    assetPlayerTest_scanning_controller.init(); 
});


var assetPlayerTest_settings_controller = {
    init:function() {

    },

    /**
     * Build and show the settings page.
     * FIXME: Need to make this a JQuery function
     */
    showSettings: function(group) {
	assetPlayerTest_settings_controller.createForm(group);
        $.mobile.changePage("#settings");
    },

    /**
     * Get the names of the available settings groups.
     * Groups are used to describe related sets of settings. When the
     * settings page is requested the user will be presented with a
     * list of links to individual settings pages for each group. If
     * no groups are defined then all settings will be displayed on a 
     * single settings page.
     * 
     * @return an array setting group names or an empty array if no groups are defined.
     */
    getGroups: function() {
	var groups = [];
	groups[0] = "about";
	groups[1] = "contact";
	return groups;
    },
	    
    /**
     * Get the settings in a given group. Note that the group names here must correspond
     * to the group names returned by assetPlayerTest_settings_controller.getSettings()
     * 
     * @return an array containing a number of Setting objects
     */
    getSettings: function(group) {
	var settings = [];
	if (group == undefined || group == "about") {
	    settings[0] = new assetPlayerTest_Setting("name", "Name", "The name you wish to be known by.", "text");
	    settings[1] = new assetPlayerTest_Setting("description", "Description", "A short description that will be shown to people viewing your details.", "textarea");
	} else if (group == "contact") {
	    settings[0] = new assetPlayerTest_Setting("homepageURL", "Homepage URL", "Your homepage URL.", "email");
	    settings[1] = new assetPlayerTest_Setting("email", "EMail", "Your email address.", "email");
	}
	return settings;
    },
    
    /**
     * Process a settings form that has been submitted by storing all the settings
     * in widget preferences. If settings groups are being used then we only process
     * settings in that group.
     * 
     * @param group the name of the group to process if groups are being used
     */
    submitSettingsForm: function(group) {
	var settings = assetPlayerTest_settings_controller.getSettings(group);
	var value;
	for (key in settings) {
	    value = $("#" + settings[key].id).attr("value");
	    widget.preferences.setItem(settings[key].id, value);
        };
        $.mobile.changePage("#home");
        $.mobile.loadPage( "settings.html" );
    },
	
    /**
     * Create the settings form and populate <div id="settings"> with it.
     * 
     * Groups are used to describe related sets of settings. When the
     * settings page is requested the user will be presented with a
     * list of links to individual settings pages for each group. If
     * no groups are defined then all settings will be displayed on a 
     * single settings page.
     * 
     * @param group the name of the group of settings we need a form to represent
     */
    createForm: function(group) {
	if (group == undefined) {
	    assetPlayerTest_settings_controller.createGroupsIndex();
	} else {
	    var settings = assetPlayerTest_settings_controller.getSettings(group);
            // FIXME: JQuerify the action	    
	    var form = $("<form action='javascript:assetPlayerTest_settings_controller.submitSettingsForm(\"" + group + "\")'>");
			
	    var title = $("<h3>Settings: " + group + "</h3>");
	    
	    for (key in settings) {
		var label = $("<label>" + settings[key].label + "</label>");
		label.attr("for", settings[key].id);
		form.append(label);
		
		var input = $("<input>");
		input.type = settings[key].type;
		input.attr("id", settings[key].id);
		input.attr("placeholder", settings[key].placeholder)
		var value = widget.preferences.getItem(settings[key].id);
		if (value != undefined) {
		    input.attr("value", value);
		}
		form.append(input);
	        
		form.append($("<br>"));
	    };
	    
	    var submit = $("<input>");
	    submit.attr("type", "submit");
	    submit.attr("value", "Save");
	    submit.attr("class", "form-button");
	    form.append(submit);
	    
            var settingsElem = $("#settings-primary");
            settingsElem.empty();
	    title.appendTo(settingsElem);
	    form.appendTo(settingsElem).trigger("create");
	}
    },
	
    /**
     * Populate the <div id="settings" page with an index to the various settings
     * groups.
     */
    createGroupsIndex: function() {
	var groups = assetPlayerTest_settings_controller.getGroups();
	
	var title = $("<h3>Settings Groups<h3>");
	
	var list = $("<ul data-role='listview'>");
	for (key in groups) {
            var item = $("<li><a href='javascript:assetPlayerTest_settings_controller.showSettings(\"" + groups[key] + "\");'>" + groups[key] + "</a></li>");
	    list.append(item);
            // FIXME: JQuerify
	    // groupLink.href = "javascript:assetPlayerTest_settings_controller.showSettings('" + groups[key] + "')"
	}
	
	var settingsElem = $("#settings-primary");
        settingsElem.empty();
	settingsElem.append(title);
	settingsElem.append(list);
    }
};

$('#home').live('pageinit',function(event) { 
  $.mobile.loadPage( "settings.html" );
  $('#showSettings').on('click',function(event){
      assetPlayerTest_settings_controller.showSettings();
  });
});

$('#settings').live('pageinit',function(event) { 
    assetPlayerTest_settings_controller.init();
});

/**
 * A setting to represent in the settings form.
 * 
 * @param id is used to identiy the setting and the form element created for it.
 * @param label is the text to place in the label for the form element.
 * @param placeholder is the text that will appear in the form element if the setting has not been set
 * @param type is the type of data to be stored in this setting, it is used to define the form element to use
 */
function assetPlayerTest_Setting(id, label, placeholder, type){
    this.id = id;
    this.label = label;
    this.placeholder = placeholder
    this.type = type;
}


$('#tile').live('pageinit',function(event) { 
    $('#tile').click(function() {
        window.widget.views.openUrl("http://wookie.apache.org/widgets/CHANGEME", null, "_blank");
    });
});



var assetPlayerTest_twoColumn_controller = {
    init:function() {
      assetPlayerTest_twoColumn_controller.update();
    },

    /**
     * Update the display
     */
    update:function() {
	// expand the search box on screens that are either wide enough (alongside results) or tall enough
	if ($.mobile.media("only screen and (min-width : 801px)")) {
	    $('#content-secondary .ui-collapsible').each(
		function() {
		    $(this).trigger('expand');
		});
	} else if ($.mobile.media("only screen and (min-width : 641px) and (max-width : 1024px)") || $.mobile.media("only screen and (min-width : 1801px)")) {
	    $('#content-secondary .ui-collapsible').each(
		function() {
		    $(this).trigger('expand');
		});
	} else {
	    $('#content-secondary .ui-collapsible').each(
		function() {
		    $(this).trigger('collapse');
		});
	}
    },

}

$('#home').live('pageinit',function(event) {
   assetPlayerTest_twoColumn_controller.init();
})



var assetPlayerTest_images_controller = { 
    init:function() { 
        var albums = {
            "places": {
                "title": "Places",
                "preview": "images/places/032.jpg",
                "assets": [
                    {
	                "src":"images/places/032.jpg"
	            },
                    {
	                "src":"images/places/042.jpg"
	            },
                    {
	                "src":"images/places/eden project.jpg"
	            },
                    {
	                "src":"images/places/edenProject.jpg"
	            }
                ]
            },
            "beeKeeping": {
                "title": "Bees",
                "preview": "images/beeKeeping/3045715262_acbb1f9b60_z.jpg",
                "assets": [
                    {
	                "src":"images/beeKeeping/3045715262_acbb1f9b60_z.jpg"
	            },
                    {
	                "src":"images/beeKeeping/BeeFindsPollen.jpg"
	            }
                ]
            },
            "vehicles": {
                "title": "Vehicles",
                "preview": "images/vehicle/11919_wpm_hires.jpg",
                "assets": [
                    {
	                "src":"images/vehicle/11919_wpm_hires.jpg"
	            },
                    {
	                "src":"images/vehicle/14021_wpm_hires.jpg"
	            },
                    {
	                "src":"images/vehicle/7863_wpm_hires.jpg"
	            },
                ]
            }
        }
	assetPlayerTest_asset_controller.setAlbums(albums);
	assetPlayerTest_scanning_controller.scanElements = $('[data-scanOrder]');
    },

};

$('#home').live('pageinit',function(event) { 
  assetPlayerTest_images_controller.init(); 
});

